from typing import List
from model.common.commonmodel import CommonModel, BuilderModel
from model.bcpnp.data import Bcpnp, Contact, General, JobOffer, ErAddress
from model.common.jobposition import PositionBase
from model.common.rcic import Rcic
from model.common.advertisement import (
    Advertisement,
    Advertisements,
    InterviewRecord,
    InterviewRecords,
    RecruitmentSummary,
)
from model.common.person import Person, PersonalAssess
from model.common.contact import Contacts
from model.common.address import Addresses
from model.common.phone import Phone, Phones
from datetime import date
from model.common.xmlfiller import XmlFiller
import os
from typing import Optional
from pdfform.bcpnp.fbemployerdeclaration import FormBuilderEmployerDeclaration
from pydantic import BaseModel


class Personal(Person):
    def __str__(self):
        return self.full_name


class Position(PositionBase):
    is_new: bool
    has_same_number: Optional[int]
    vacancies_number: Optional[int]
    laidoff_with12: Optional[int]
    laidoff_current: Optional[int]


class EmployerDeclaratonFormModel(BaseModel, BuilderModel):
    eraddress: List[ErAddress]
    phone: List[Phone]
    general: General
    contact: List[Contact]
    position: Position
    personal: Personal
    joboffer: JobOffer
    personalassess: PersonalAssess
    # bcpnp: Bcpnp
    rcic: Rcic
    advertisement: List[Advertisement]
    interviewrecord: List[InterviewRecord]
    recruitmentsummary: RecruitmentSummary

    @property
    def work_location(self):
        addresses = Addresses(self.eraddress)
        return addresses.working

    @property
    def phones(self):
        return Phones(self.phone)

    @property
    def selected_contact(self):
        contacts = Contacts(self.contact)
        return contacts.preferredContact

    @property
    def interviews(self):
        return InterviewRecords(self.interviewrecord)

    @property
    def advertisements(self):
        return Advertisements(self.advertisement)

    @property
    def businessaddress(self):
        eraddress = Addresses(self.eraddress)
        return eraddress.business

    @property
    def mailingaddress(self):
        eraddress = Addresses(self.eraddress)
        return eraddress.mailing

    @property
    def person(self):
        return {
            "first_name": self.personal.first_name,
            "last_name": self.personal.last_name,
            "full_name": self.personal.full_name,
            "attributive": self.personal.attributive,
            "object": self.personal.object,
            "subject": self.personal.subject,
            "short_name": self.personal.short_name,
            "why_tfw": self.personalassess.why_qualified_say,
        }

    def context(self, *args, **kwargs):
        pass

    def make_pdf_form(self, output_json, *args, **kwargs):
        pf = FormBuilderEmployerDeclaration(self)
        pf.save(output_json)

    def make_web_form(self, *args, **kwargs):
        pass


class EmployerDeclaratonFormModelE(CommonModel, EmployerDeclaratonFormModel):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = [
            "excel/er.xlsx",
            "excel/pa.xlsx",
            "excel/recruitment.xlsx",
            "excel/bcpnp.xlsx",
            "excel/rep.xlsx",
        ]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from datetime import date
from typing import Optional, List
from pydantic import BaseModel, root_validator, EmailStr
from model.common.commonmodel import CommonModel
from model.common.address import Address
from model.common.contact import ContactBase
from termcolor import colored
from model.common.jobposition import PositionBase
from model.common.jobofferbase import JobofferBase
from model.common.person import Person, PersonalAssess


class LmiaCase(BaseModel):
    area_index: int
    province_index: int
    unemploy_rate: float
    area_median_wage: float
    noc_outlook: int
    reason_failed_hire_canadian: Optional[str]
    provincial_median_wage: float
    is_in_10_days_priority: Optional[bool]
    top10_wages: float
    is_waived_from_advertisement: bool
    purpose_of_lmia: str
    stream_of_lmia: str
    has_another_employer: bool
    another_employer_name: Optional[str]
    number_of_tfw: int
    duration_number: float
    duration_unit: str
    duration_reason: str
    has_attestation: bool
    is_waived_from_advertisement: bool
    reason_for_waived: Optional[str]
    provide_details_even_waived: Optional[str]
    use_jobbank: Optional[bool]
    reason_not_use_jobbank: Optional[str]


class Position(BaseModel):
    why_hire: str
    who_current_fill: str
    how_did_you_find: str
    how_when_offer: str
    worked_working: bool
    worked_working_details: Optional[str]


class Joboffer(JobofferBase):
    license_request: bool
    license_description: Optional[str]
    union: bool
    payment_way: str
    ot_after_hours_unit: Optional[str]
    ot_after_hours: Optional[float]
    is_working: bool
    has_probation: bool
    probation_duration: Optional[str]
    offer_date: date
    duties_brief: str
    duties: str
    english_french: bool
    oral: Optional[str]
    writing: Optional[str]
    reason_for_no: Optional[str]
    other_language_required: bool
    reason_for_other: Optional[str]
    education_level: str
    specific_edu_requirement: Optional[str]
    skill_experience_requirement: Optional[str]
    other_requirements: Optional[str]

    @root_validator
    def checkProbation(cls, values):
        has_probation = values.get("has_probation", None)
        probation_duration = values.get("probation_duration")
        if has_probation and not probation_duration:
            raise ValueError(
                "Since has probation is true, but did not input probation duration"
            )
        return values

    @root_validator
    def checkLanguage(cls, values):
        english_french = values.get("english_french", None)
        oral = values.get("oral", None)
        writing = values.get("writing", None)
        reason_for_no = values.get("reason_for_no", None)
        if english_french and (not oral or not writing):
            raise ValueError(
                "Since English or French is true, but did not input oral or/and writting language requirement"
            )
        if not english_french and not reason_for_no:
            raise ValueError(
                "Since there is no English or French requirement, but did not input the reason"
            )
        return values

    @root_validator
    def checkOtherLanguageReason(cls, values):
        other_language_required = values.get("other_language_required", None)
        reason_for_other = values.get("reason_for_other", None)
        if other_language_required and not reason_for_other:
            raise ValueError(
                "Since required other language in job offer sheet,but did not input the reason"
            )
        return values


class Lmi(BaseModel):
    brief_benefit: str
    job_creation_benefit: Optional[str]
    skill_transfer_benefit: Optional[str]
    fill_shortage_benefit: Optional[str]
    other_benefit: Optional[str]
    canadian_lost_job: bool
    canadian_lost_job_info: Optional[str]
    safety_concerns: str

    @root_validator
    def checkLabourDisput(cls, values):
        canadian_lost_job = values.get("canadian_lost_job", None)
        canadian_lost_job_info = values.get("canadian_lost_job_info", None)
        if canadian_lost_job and not canadian_lost_job_info:
            raise ValueError(
                "Since Canadian will lost job because of hiring TFW,but did not input the details about it."
            )
        return values


class LmiaRcic(BaseModel):
    lmiacase: LmiaCase
    position: Position
    joboffer: Joboffer
    lmi: Lmi
    personalassess: PersonalAssess


class LmiaRcicE(CommonModel, LmiaRcic):
    lmiacase: LmiaCase
    position: Position
    joboffer: Joboffer
    lmi: Lmi
    personalassess: PersonalAssess

    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/er.xlsx", "excel/lmia.xlsx", "excel/pa.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from pdfform.tr.common import TrCommon
from pdfform.tr import option_lists


class FormBuilder5709(TrCommon):
    """Form builder"""

    def start(self):
        """Add initial 3 skips"""
        self.form.add_skip(3)

    def add_header(self):
        """Add header section items"""
        personal = self.applicant.personal
        self.form.add_text(personal.uci)
        self.form.add_dropdown(
            self.applicant.trcasein.service_in, option_lists.service_language
        )

        if (
            self.applicant.spincanada.application_purpose
            == "apply or extend study permit"
        ):
            self.form.add_checkbox(True)
        else:
            self.form.add_skip(1)
        if self.applicant.spincanada.application_purpose == "restore status as student":
            self.form.add_checkbox(True)
        else:
            self.form.add_skip(1)
        if self.applicant.spincanada.application_purpose == "TRP":
            self.form.add_checkbox(True)
        else:
            self.form.add_skip(1)

    def add_study_detail(self):
        """Add study details"""
        self.form.add_info("details of study section")

        detail = self.applicant.spincanada
        self.form.add_text(detail.school_name)
        self.form.add_dropdown(
            option_lists.get_5709_study_level(detail.study_level),
            option_lists.level_of_study,
        )
        self.form.add_dropdown(detail.study_field, option_lists.field_of_study, True)

        self.form.add_dropdown(detail.province, option_lists.canada_province)
        self.form.add_dropdown(
            detail.city.upper(), option_lists.canada_province_cities[detail.province]
        )
        self.form.add_text(detail.address)
        self.form.add_text(detail.dli)
        self.form.add_text(detail.student_id)
        self.form.add_date(detail.start_date)
        self.form.add_date(detail.end_date)

        self.form.add_text(detail.tuition_cost)
        self.form.add_text(detail.room_cost)
        self.form.add_text(detail.other_cost)
        self.form.add_text(detail.fund_available)
        self.form.add_dropdown(detail.paid_person, option_lists.expense_by)
        self.form.add_skip(1)
        self.form.add_radio(detail.apply_work_permit)
        if detail.apply_work_permit:
            self.form.add_dropdown(
                detail.work_permit_type, option_lists.work_permit_student
            )
            self.form.add_text(detail.caq_number)
            self.form.add_text(detail.expiry_date)
        else:
            self.form.add_skip(3)

    def get_form(self):
        self.start()
        self.add_header()
        self.add_personal_detail()
        self.add_language()
        self.add_passport()
        self.add_national_id()
        self.add_uspr_card()
        self.add_contact_information(form_type="5708")
        self.add_coming_into_canada()
        self.add_study_detail()
        self.add_education()
        self.add_employment()
        self.add_background()
        self.add_signature()
        return self.form



from webform.webcommon import WebPages

# 1. EE
class EE(WebPages):
    @property
    def actions(self):
        return [self.page1, self.page2, self.page3]

    @property
    def page1(self):
        support_what = (
            "#\\39 340 > div > input:nth-child(1)"
            if self.app.lmiacase.purpose_of_lmia == "Support Permanent Resident only"
            else "#\\39 340 > div > input:nth-child(4)"
        )
        joined_employer = (
            "#\39 341 > div > input:nth-child(1)"
            if self.app.lmiacase.has_another_employer
            else "#\\39 341 > div > input:nth-child(4)"
        )
        page_actions = [
            self.web_element.radioElement(
                support_what, label="Support PR only or WP&PR"
            ),
            self.web_element.radioElement(
                joined_employer, label="Supported by another employer?"
            ),
        ]

        return self.web_element.pageElement(
            "#next", "#\\39 342 > textarea", page_actions, label="LMIA purpose"
        )

    @property
    def page2(self):
        has_worked_is_working = (
            "#\\39 344 > div > input:nth-child(1)"
            if self.app.position.worked_working
            else "#\\39 344 > div > input:nth-child(4)"
        )
        page_actions = [
            self.web_element.areatextElement(
                "#\\39 342 > textarea",
                self.app.position.who_current_fill,
                label="Who is currently filling in this position",
            ),
            self.web_element.areatextElement(
                "#\\39 343 > textarea",
                self.app.position.how_did_you_find,
                label="How did you find?",
            ),
            self.web_element.radioElement(
                has_worked_is_working,
                label="Has previously employed or is currently employed",
            ),
        ]
        next_page_tag="#\\39 345 > textarea" if self.app.position.worked_working else "#\\39 346 > textarea "
        return self.web_element.pageElement(
            "#next", next_page_tag, page_actions, label="PR position info"
        )

    @property
    def page3(self):
        has_someone_in_same_position_id = (
            "#\\39 433 > div > input:nth-child(1)"
            if self.app.position.has_same
            else "#\\39 433 > div > input:nth-child(4)"
        )
        page_actions=[
            self.web_element.areatextElement(
                "#\\39 345 > textarea" ,
                self.app.position.worked_working_details,
                label="Worked details",
            )
        ] if self.app.position.worked_working else []
        
        page_actions += [
            self.web_element.areatextElement(
                "#\\39 346 > textarea" ,
                self.app.personalassess.why_qualified_say,
                label="Why the TFW qualfied",
            ),
            self.web_element.areatextElement(
                "#\\39 347 > textarea",
                self.app.position.how_when_offer,
                label="How and when did you offer the job",
            ),
            self.web_element.radioElement(
                has_someone_in_same_position_id,
                label="Has other employee in the same position?",
            ),
        ]
        next_page_tag = "#\\39 438 > div > input:nth-child(4)"  # has atypical schedule
        return self.web_element.pageElement(
            "#next", next_page_tag, page_actions, label="PR position info"
        )



from webform.formcontrol import WebElement


class Login:
    def __init__(self, account, password, security_answers: dict):
        self.account = account
        self.password = password
        self.security_answers = security_answers
        self.web_element = WebElement()

    @property
    def actions(self):
        return [self.login, self.securityCheck, self.acknowlege]

    @property
    def login(self):

        url = "https://tfwp-jb.lmia.esdc.gc.ca/employer/"
        next_page_tag1 = "#loginForm\\:j_id_47"

        login_id = "#loginForm\\:j_id_47"
        account_id = "#loginForm\\:input-email"
        password_id = "#loginForm\\:input-password"
        next_page_tag2 = "#continueButton"

        actions = [
            self.web_element.gotoPageElement(url, wait_for=next_page_tag1),
            self.web_element.loginElement(
                account_id, self.account, password_id, self.password
            ),
        ]

        return self.web_element.pageElement(
            login_id, next_page_tag2, actions, label="Login"
        )

    @property
    def securityCheck(self):
        security_label_id = "#securityForm > fieldset > div > div.form-group.has-show.has-feedback > label > span.field-name"
        security_answer_id = "#securityForm\\:input-security-answer"
        actions = [
            self.web_element.securityElement(
                security_label_id,
                security_answer_id,
                self.security_answers,
            )
        ]
        continue_button = "#continueButton"
        next_page_tag = "#modal-accept"
        return self.web_element.pageElement(
            continue_button, next_page_tag, actions, label="Security question check"
        )

    @property
    def acknowlege(self):
        accept_button = "#modal-accept"
        next_page_tag = "#wb-auto-6_filter > label > input[type=search]"  # Filter
        return WebElement().pageElement(
            accept_button, next_page_tag, [], label="Acknowlege important message"
        )



from pydantic import BaseModel, EmailStr, root_validator
from typing import Optional, List
from datetime import date

from model.common.commonmodel import CommonModel


class Family(BaseModel):
    relationship: Optional[str]
    last_name: Optional[str]
    first_name: Optional[str]
    native_last_name: Optional[str]
    native_first_name: Optional[str]
    date_of_birth: Optional[date]
    date_of_death: Optional[date]
    place_of_birth: Optional[str]
    birth_country: Optional[str]
    marital_status: Optional[str]
    email: Optional[EmailStr]
    address: Optional[str]

    @root_validator
    def checkFields(cls, values):
        last_name = values.get("last_name")
        first_name = values.get("first_name")
        native_last_name = values.get("native_last_name")
        native_first_name = values.get("native_first_name")
        date_of_birth = values.get("date_of_birth")
        place_of_birth = values.get("place_of_birth")
        birth_country = values.get("birth_country")
        marital_status = values.get("marital_status")
        address = values.get("address")

        if (
            last_name
            and first_name
            and not all(
                [
                    native_last_name,
                    native_first_name,
                    date_of_birth,
                    place_of_birth,
                    birth_country,
                    marital_status,
                    address,
                ]
            )
        ):
            raise ValueError(
                f"Some fields of {first_name} {last_name} in info-family sheet are not completed, please check"
            )
        return values


class M5406Model(BaseModel):
    family: List[Family]


class M5406ModelE(CommonModel, M5406Model):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/pa.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from .context import DATADIR
from typing import List

from pydantic import EmailStr, BaseModel
from model.common.commonmodel import CommonModel
from model.bcpnp.data import Bcpnp, General, JobOffer, ErAddress
from model.common.jobposition import PositionBase
from model.common.rcic import Rcic
from model.common.advertisement import Advertisement, Advertisements, RecruitmentSummary
from model.common.person import Person, PersonalAssess
from model.common.address import Address, Addresses
from model.common.phone import Phone, Phones
from model.common.wordmaker import WordMaker
import os
from model.common.commonmodel import CommonModel, BuilderModel


class Personal(Person):
    email: EmailStr

    def __str__(self):
        return self.full_name


class Position(PositionBase):
    pass


class EmployeeTrainingModel(BaseModel, BuilderModel):
    eraddress: List[ErAddress]
    general: General
    position: Position
    personal: Personal
    phone: List[Phone]
    address: List[Address]
    joboffer: JobOffer
    personalassess: PersonalAssess
    bcpnp: Bcpnp
    rcic: Rcic
    advertisement: List[Advertisement]
    recruitmentsummary: RecruitmentSummary

    @property
    def residential_address(self):
        addresses = Addresses(self.address)
        return addresses.residential

    @property
    def work_location(self):
        addresses = Addresses(self.eraddress)
        return addresses.working

    @property
    def selected_contact(self):
        contacts = Contacts(self.contact)
        return contacts.preferredContact

    @property
    def advertisements(self):
        return Advertisements(self.advertisement)

    @property
    def phone_number(self):
        return Phones(self.phone).PreferredPhone

    @property
    def person(self):
        return {
            "full_name": self.personal.full_name,
            "attributive": self.personal.attributive,
            "object": self.personal.object,
            "subject": self.personal.subject,
            "short_name": self.personal.short_name,
            "email": self.personal.email,
            "phone": self.phone_number,
            "address": self.residential_address,
        }

    def context(self, *args, **kwargs):
        context = {
            **self.dict(),
            "advertisement": self.advertisements,
            "personal": self.person,
            "date_of_offer": self.joboffer.date_of_offer,
            "work_start_date": self.joboffer.start_date_say,
            "joboffer_date": self.joboffer.date_of_offer,
            "work_location": self.work_location,
        }
        return context

    def make_pdf_form(self, *args, **kwargs):
        pass

    def make_web_form(self, *args, **kwargs):
        pass


class EmployeeTrainingModelE(CommonModel, EmployeeTrainingModel):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = [
            "excel/er.xlsx",
            "excel/pa.xlsx",
            "excel/recruitment.xlsx",
            "excel/bcpnp.xlsx",
            "excel/rep.xlsx",
        ]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from webform.webcommon import WebPages
from webform.lmiaportal.wage import Wage
from webform.lmiaportal.foreignnational import ForeignNational

# 1. Common part of High Wage stream and Low Wage Stream
class HWS_LWS(WebPages):
    @property
    def actions(self):
        return [self.page1, *self.page2, *self.page3]

    @property
    def page1(self):
        """Check if Quebec's facilitated process.
        We assume that we don't do Quebec LMIA, so here are all answer no
        """
        answer_no = self.web_element.radioElement(
            "#\\39 138 > div > input:nth-child(4)", label="No"
        )
        next_page_tag = "#\\39 159 > div > input:nth-child(4)"  # wage converted? No id
        return self.web_element.pageElement(
            "#next",
            next_page_tag,
            [answer_no],
            label="No for Quebec facilitated process",
        )

    @property
    def page2(self):
        """Wage and work location, which is same as EE"""
        return Wage(self.app).actions

    @property
    def page3(self):
        """Provide TFW's name? and if so, TFW's details"""
        named = (
            self.app.lmiacase.stream_of_lmia == "LWS" and self.app.emp5627.named
        ) or (self.app.lmiacase.stream_of_lmia == "HWS" and self.app.emp5626.named)

        provide_tfw_name_id = (
            "#\\39 180 > div > input:nth-child(1)"
            if named
            else "#\\39 180 > div > input:nth-child(4)"
        )
        provide_tfw_name = self.web_element.radioElement(
            provide_tfw_name_id,
            label="Yes" if named else "No",
        )

        if named:
            next_page_tag = (
                "#\\39 181 > div > input:nth-child(4)"  # use online feature No id
            )
            provide_name_page = self.web_element.pageElement(
                "#next", next_page_tag, [provide_tfw_name], label="Provide TFW's name?"
            )
            foreign_national_pages = ForeignNational(self.app).actions
            return [provide_name_page, *foreign_national_pages]
        else:
            # provide accomadation No id if LWS, else if in seasonal occupation if HWS
            next_page_tag = (
                "#\\39 378 > div > input:nth-child(4)"
                if self.app.lmiacase.stream_of_lmia == "LWS"
                else "#\\39 349 > div > input:nth-child(4)"
            )
            return [
                self.web_element.pageElement(
                    "#next",
                    next_page_tag,
                    [provide_tfw_name],
                    label="Provide TFW's name?",
                )
            ]



from webform.webcommon import WebPages, Page

#  Transition plan
class TransitionPlan(WebPages):
    @property
    def actions(self):
        tp_waivable = True if self.app.emp5626.tp_waivable else False
        return (
            [self.page1, self.page2, self.pages_after_if_exempted]
            if tp_waivable
            else [self.page1, self.page2, self.completed_tp, self.transition_plan]
        )

    @property
    def page1(self):
        """Is this application for a seasonal occupation?"""
        is_seasonal = True if self.app.emp5626.is_in_seasonal_industry else False
        for_seasonal_id = (
            "#\\39 349 > div > input:nth-child(1)"
            if is_seasonal
            else "#\\39 349 > div > input:nth-child(4)"
        )
        for_seasonal = self.web_element.radioElement(
            for_seasonal_id, label="Yes" if is_seasonal else "No"
        )
        next_page_tag = "#\\39 358 > div > input:nth-child(4)"  # wish to consider be exempted? No id
        return Page(
            [for_seasonal],
            "#next",
            next_page_tag,
            label="Is this for a seasonal occupation?",
        ).page

    @property
    def page2(self):
        is_seasonal = True if self.app.emp5626.is_in_seasonal_industry else False
        # if tp waived, hour and pay has same position no id
        # if not, have you completed a TP? No id
        next_page_tag = (
            "#\\39 433 > div > input:nth-child(4)"
            if self.app.emp5626.tp_waivable
            else "#\\39 362 > div > input:nth-child(4)"
        )
        if is_seasonal:
            return Page(
                [*self.seasonal_position, *self.hw_tp, self.do_you_wish],
                "#next",
                next_page_tag,
                label="Seasonal positions and high wage Transition Plan",
            ).page
        else:
            return Page(
                [*self.hw_tp, self.do_you_wish],
                "#next",
                next_page_tag,
                label="High wage Transition Plan",
            ).page

    @property
    def seasonal_position(self):
        how_many_canadians_past = self.web_element.inputElement(
            "#\\39 354 > input",
            str(self.app.emp5626.last_canadian_number),
            label="How many Canadians last peak period?",
        )
        how_many_tfws_past = self.web_element.inputElement(
            "#\\39 355 > input",
            str(self.app.emp5626.last_tfw_number),
            label="How many TFWs last peak period?",
        )
        from_month = self.web_element.selectElement(
            "#\\39 352 > select",
            self.app.emp5626.start_month,
            label="Start month of last peak season",
        )
        to_month = self.web_element.selectElement(
            "#\\39 353 > select",
            self.app.emp5626.end_month,
            label="End month of last peak season",
        )
        return [how_many_canadians_past, how_many_tfws_past, from_month, to_month]

    @property
    def hw_tp(self):
        how_many_canadians = self.web_element.inputElement(
            "#\\39 356 > input",
            str(self.app.emp5626.current_canadian_number),
            label="How many Canadians currently?",
        )
        how_many_tfws = self.web_element.inputElement(
            "#\\39 357 > input",
            str(self.app.emp5626.current_tfw_number),
            label="How many TFWs currently?",
        )
        return [how_many_canadians, how_many_tfws]

    @property
    def do_you_wish(self):
        tp_waivable = True if self.app.emp5626.tp_waivable else False
        consider_exempted_id = (
            "#\\39 358 > div > input:nth-child(1)"
            if tp_waivable
            else "#\\39 358 > div > input:nth-child(4)"
        )
        consider_exempted = self.web_element.radioElement(
            consider_exempted_id, label="Yes" if tp_waivable else "No"
        )
        # if waivable, goto hours and pay has same position, No id
        # else, goto "have you completed a TP?" id No
        # next_page_tag = (
        #     "#\\39 433 > div > input:nth-child(4)"
        #     if tp_waivable
        #     else "#\\39 362 > div > input:nth-child(4)"
        # )
        # return Page(
        #     [consider_exempted],
        #     "#next",
        #     next_page_tag,
        #     label="Is the application wavied from Transition Plan?",
        # ).page
        return consider_exempted

    # exempted functions
    @property
    def pages_after_if_exempted(self):
        page_actions = [
            self.exemption_criteria,
            self.details_support_exemption,
            self.hours_pay,
        ]
        next_page_tag = (
            "#\\39 438 > div > input:nth-child(4)"  # atypical schedule? No id
        )
        return Page(
            page_actions,
            "#next",
            next_page_tag,
            label="Exemption criteria and has same position",
        ).page

    @property
    def exemption_criteria(self):
        criteria_types = {
            "Caregiver positions in health care institutions": "#\\39 360 > div > input:nth-child(1)",
            "Limited duration positions": "#\\39 360 > div > input:nth-child(4)",
            "On-farm primary agricultural positions": "#\\39 360 > div > input:nth-child(7)",
            "Positions within a specialized occupation": "#\\39 360 > div > input:nth-child(10)",
            "Unique skills or traits": "#\\39 360 > div > input:nth-child(13)",
        }
        return self.web_element.radioElement(
            criteria_types.get(self.app.emp5626.waive_creteria),
            label="Which transition plan exemption?",
        )

    @property
    def details_support_exemption(self):
        return self.web_element.areatextElement(
            "#\\39 361 > textarea",
            self.app.emp5626.waive_creteria,
            label="Details to support the exemption",
        )

    # not exempted functions
    @property
    def completed_tp(self):
        is_completed_id = (
            "#\\39 362 > div > input:nth-child(1)"
            if self.app.emp5626.has_finished_tp
            else "#\\39 362 > div > input:nth-child(4)"
        )
        is_completed = self.web_element.radioElement(
            is_completed_id, label="Yes" if self.app.emp5626.has_finished_tp else "No"
        )
        next_page_tag = "#\\39 433 > div > input:nth-child(4)"  # Hours and pay has same position NO id
        return Page(
            [is_completed],
            "#next",
            next_page_tag,
            label="Have you completed a Transition Plan?",
        ).page

    @property
    def transition_plan(self):
        actions = [self.pre_tp_description] if self.app.emp5626.has_finished_tp else []
        actions.extend([*self.add_tps, self.hours_pay])
        next_page_tag = (
            "#\\39 438 > div > input:nth-child(4)"  # atypical schedule id no
        )
        return Page(actions, "#next", next_page_tag, label="Transition Plan").page

    @property
    def pre_tp_description(self):
        return self.web_element.areatextElement(
            "#\\39 364 > textarea",
            self.app.emp5626.finished_tp_result,
            label="Previous transition plan activity description",
        )

    def add_tp(self, index):
        add = self.web_element.buttonElement("#addActivities", label="Add")
        wait_for_element = self.web_element.waitForElement(
            "#\\39 373 > textarea"
        )  # wait for new elements appear, additional employer comments id

        title = self.web_element.inputElement(
            "#\\39 370 > input",
            getattr(self.app.emp5626, f"activity{index}_title"),
            label="Title",
        )
        description = self.web_element.areatextElement(
            "#\\39 371 > textarea",
            getattr(self.app.emp5626, f"activity{index}_description"),
            label="Activity description",
        )
        outcome = self.web_element.areatextElement(
            "#\\39 372 > textarea",
            getattr(self.app.emp5626, f"activity{index}_outcome"),
            label="Activity outcome",
        )
        comment = self.web_element.areatextElement(
            "#\\39 373 > textarea",
            getattr(self.app.emp5626, f"activity{index}_comment"),
            label="Employer comment",
        )

        save = self.web_element.buttonElement("#saveActivities", label="Save")
        wait1s = self.web_element.waitElement(1000)
        return [
            add,
            wait_for_element,
            title,
            description,
            outcome,
            comment,
            save,
            wait1s,
        ]

    @property
    def add_tps(self):
        tps = []
        for i in range(1, 6):
            tp = self.add_tp(i)
            tps.extend(tp)

        confirm = self.web_element.checkboxElement(
            "#\\39 376 > input", True, label="I have read"
        )
        tps.append(confirm)
        return tps

    @property
    def hours_pay(self):
        has_same_id = (
            "#\\39 433 > div > input:nth-child(1)"
            if self.app.position.has_same
            else "#\\39 433 > div > input:nth-child(4)"
        )
        return self.web_element.radioElement(
            has_same_id, label="Has same employee in the position"
        )



from abc import ABC, abstractmethod
from webform.formcontrol import WebElement


class WebPages(ABC):
    """Base class for a set of pages"""

    def __init__(self, app: object):
        self.app = app
        self.web_element = WebElement()

    @abstractmethod
    def actions(self):
        """page actions"""


class Page(ABC):
    """Base class for a single page"""

    def __init__(self, page_actions, id, next_page_tag, label="Page") -> None:
        self.page_actions = page_actions
        self.id = id
        self.next_page_tag = next_page_tag
        self.label = label

    @property
    def page(self):
        return WebElement().pageElement(
            self.id, self.next_page_tag, self.page_actions, label=self.label
        )



from termcolor import colored
from functools import reduce
from typing import List, Optional
from model.common.address import Address
from ..common.educationbase import EducationHistory
from model.common.phone import Phone
from model.common.trperson import (
    COR,
    PersonId,
    Personal,
    Marriage,
    Education,
    Employment,
    Travel,
    Family,
)
from model.common.person import PersonalAssess
from model.common.tr import TrCase, Wp, TrBackground
from model.common.commonmodel import CommonModel, BuilderModel
from pdfform.tr.fb1295 import FormBuilder1295
from model.lmia.common import Rcic
import json
from pydantic import BaseModel


class M1295Model(BaseModel, BuilderModel):
    personal: Personal
    personalassess: PersonalAssess
    marriage: Marriage
    personid: List[PersonId]
    address: List[Address]
    education: List[Education]
    employment: List[Employment]
    travel: List[Travel]
    family: List[Family]
    phone: List[Phone]
    cor: List[COR]
    trcase: TrCase
    wp: Wp
    trbackground: TrBackground
    rcic: Rcic

    def make_pdf_form(self, output_json, *args, **kwargs):
        pf = FormBuilder1295(self)
        form = pf.get_form()
        with open(output_json, "w") as output:
            json.dump(form.actions, output, indent=3, default=str)
        print(colored(f"{output_json} has been created. ", "green"))

    def make_web_form(self, output_json, upload_dir, rcic, *args, **kwargs):
        raise ValueError("This model doesn't have webform...")

    def context(self, *args, **kwargs):
        education = EducationHistory(self.education)
        educations = (
            education.post_secondary
            if len(education.post_secondary) > 0
            else education.high_school
        )
        return {
            **self.dict(),
            "birthday": self.personal.birthday,
            "respectful_full_name": self.personal.respectful_full_name,
            "short_name": self.personal.short_name,
            "educations": educations,
            "ties": [self.wp.family_tie, self.wp.economic_tie, self.wp.other_tie],
        }


class M1295ModelE(CommonModel, M1295Model):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/tr.xlsx", "excel/pa.xlsx", "excel/rep.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



# from context import BASEDIR
import argparse
from tabulate import tabulate
from termcolor import colored
import os
import json
import requests
from dotenv import dotenv_values
from assess.engine.quickassessmodel import QuickAssessModel
from datetime import date, datetime
import warnings

warnings.filterwarnings("ignore")

api_url = "https://jackyzhang.pro/assess/quick/"


def education_level(level):
    edu_level = {
        "Doctor": ["Doctor"],
        "Master": ["Master"],
        "Bachelor": ["Post-graduate diploma", "Bachelor"],
        "Diploma": ["Associate", "Diploma/Certificate"],
        "Secondary": ["High School"],
        "Other": ["Less than high school"],
    }
    for el, l in edu_level.items():
        if level in l:
            return el


def getData(excel_file):
    data = QuickAssessModel(excels=[excel_file]).dict()
    data = data["facts"]
    level = data["level"]

    return_data = {
        "basic": {
            "dob": data["dob"].strftime("%Y-%m-%d"),
            "email": "jacky@gmail.com",
            "phone": "7783215110",
            "lastName": "Zhang",
            "firstName": "Jacky",
            "birthCity": "Nanjing",
            "birthCountry": "China",
        },
        "language": {
            "test_date": date.today().strftime("%Y-%m-%d"),
            "reading": data["reading"] if data["reading"] else 0,
            "writting": data["writting"] if data["writting"] else 0,
            "speaking": data["speaking"] if data["speaking"] else 0,
            "listening": data["listening"] if data["listening"] else 0,
            "testFormat": data["test_format"] if data["test_format"] else "IELTS",
        },
        "education": {
            "level": education_level(level),
            "country": data["edu_country"],
            "province": data["edu_province"],
            "institute": "NJU",
            "graduateDate": data["graduate_date"].strftime("%Y-%m-%d"),
        },
        "family": [
            {
                "dob": data["relative_dob"].strftime("%Y-%m-%d")
                if data["relative_dob"]
                else datetime.today().strftime("%Y-%m-%d"),
                "city": "Vancouver",
                "name": "Li",
                "province": data["relative_province"]
                if data["relative_province"]
                else "BC",
                "canadaStatus": data["canada_status"]
                if data["canada_status"]
                else "PR",
                "relationship": data["relationship"]
                if data["relationship"]
                else "spouse",
            }
        ],
        "assets": {
            "liquidAssets": data["liquid_assets"] if data["liquid_assets"] else 0,
            "netWorth": data["net_worth"] if data["net_worth"] else 0,
        },
        "workExperience": {
            "startDate": data["start_date"].strftime("%Y-%m-%d"),
            "endDate": data["end_date"].strftime("%Y-%m-%d"),
            "noc": data["work_noc"],
            "term": data["term"],
            "country": data["work_country"],
            "jobTitle": data["job_title"],
            "province": data["work_province"],
            "sharePercentage": data["share_percentage"],
        },
        "adaptation": {
            "noc": data["joboffer_noc"] if data["joboffer_noc"] else "1123",
            "title": "Marketing specialist",
            "jobOffer": data["job_offer"],
            "province": data["joboffer_province"]
            if data["joboffer_province"]
            else "BC",
        },
    }
    return return_data


def getSolutions(data):
    path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".env"))
    config = dotenv_values(path)
    token = config["TOKEN"]
    headers = {"Authorization": f"Token {token}", "Content-Type": "application/json"}
    response = requests.post(api_url, json=data, headers=headers)
    if response.status_code == 200:
        return response.json()
    else:
        print(
            f" Response status code is {response.status_code}, and there is no feedback"
        )


def showData(solutions, args):
    if solutions:
        print(
            colored(
                "You may be qualified for the Canadian immigration programs as the following\n",
                "green",
            )
        )
        for prog in solutions:
            print(colored("* {program} *".format(**prog), "green"))
            print(
                colored("Stream: ", "green"),
                colored("{stream}".format(**prog), "white"),
            )
            if args.description:
                print(
                    colored("Description: ", "green"),
                    colored("{description}".format(**prog), "white"),
                )
            if args.remark:
                print(
                    colored("Remark: ", "green"),
                    colored("{remark}".format(**prog), "white"),
                )
    else:
        print(colored("You are not qualified for any program", "red"))


def main():
    parser = argparse.ArgumentParser(
        description="used for processing everything noc related"
    )
    parser.add_argument("-e", "--excel", help="input excel file")
    parser.add_argument("-j", "--json", help="print json data ", action="store_true")
    parser.add_argument(
        "-d",
        "--description",
        help="print program description data ",
        action="store_true",
    )
    parser.add_argument(
        "-r", "--remark", help="print program remark data ", action="store_true"
    )

    args = parser.parse_args()

    if args.excel:
        data = getData(args.excel)
        solutions = getSolutions(data)
        showData(solutions, args)
        if args.json:
            print(json.dumps(data, indent=3))


if __name__ == "__main__":
    main()
